@@ -94,9 +94,7 @@ static void output_handle_frame(struct wl_listener *listener, void *data) {
 	int width, height;
 	wlr_output_effective_resolution(output->wlr_output, &width, &height);
 
-	struct wlr_render_pass *pass = wlr_output_begin_render_pass(output->wlr_output, &output_state,
-		NULL, NULL);
-
+	struct wlr_render_pass *pass = wlr_output_begin_render_pass(output->wlr_output, &output_state, NULL);
 	wlr_render_pass_add_rect(pass, &(struct wlr_render_rect_options){
 		.box = { .width = width, .height = height },
 		.color = { 0.3, 0.3, 0.3, 1 },
