@@ -6,7 +6,6 @@
 #include <wlr/types/wlr_xdg_output_v1.h>
 #include <wlr/util/log.h>
 #include "xdg-output-unstable-v1-protocol.h"
-#include "util/signal.h"
 
 #define OUTPUT_MANAGER_VERSION 3
 #define OUTPUT_DONE_DEPRECATED_SINCE_VERSION 3
@@ -128,11 +127,11 @@ static void output_manager_handle_get_xdg_output(struct wl_client *client,
 		wl_resource_get_link(xdg_output_resource));
 
 	// Name and description should only be sent once per output
-	uint32_t version = wl_resource_get_version(xdg_output_resource);
-	if (version >= ZXDG_OUTPUT_V1_NAME_SINCE_VERSION) {
+	uint32_t xdg_version = wl_resource_get_version(xdg_output_resource);
+	if (xdg_version >= ZXDG_OUTPUT_V1_NAME_SINCE_VERSION) {
 		zxdg_output_v1_send_name(xdg_output_resource, output->name);
 	}
-	if (version >= ZXDG_OUTPUT_V1_DESCRIPTION_SINCE_VERSION &&
+	if (xdg_version >= ZXDG_OUTPUT_V1_DESCRIPTION_SINCE_VERSION &&
 			output->description != NULL) {
 		zxdg_output_v1_send_description(xdg_output_resource,
 			output->description);
@@ -140,7 +139,11 @@ static void output_manager_handle_get_xdg_output(struct wl_client *client,
 
 	output_send_details(xdg_output, xdg_output_resource);
 
-	wl_output_send_done(output_resource);
+	uint32_t wl_version = wl_resource_get_version(output_resource);
+	if (wl_version >= WL_OUTPUT_DONE_SINCE_VERSION &&
+			xdg_version >= OUTPUT_DONE_DEPRECATED_SINCE_VERSION) {
+		wl_output_send_done(output_resource);
+	}
 }
 
 static const struct zxdg_output_manager_v1_interface
@@ -189,7 +192,7 @@ static void handle_output_description(struct wl_listener *listener,
 
 static void add_output(struct wlr_xdg_output_manager_v1 *manager,
 		struct wlr_output_layout_output *layout_output) {
-	struct wlr_xdg_output_v1 *output = calloc(1, sizeof(struct wlr_xdg_output_v1));
+	struct wlr_xdg_output_v1 *output = calloc(1, sizeof(*output));
 	if (output == NULL) {
 		return;
 	}
@@ -227,7 +230,15 @@ static void handle_layout_change(struct wl_listener *listener, void *data) {
 }
 
 static void manager_destroy(struct wlr_xdg_output_manager_v1 *manager) {
-	wlr_signal_emit_safe(&manager->events.destroy, manager);
+	struct wlr_xdg_output_v1 *output, *tmp;
+	wl_list_for_each_safe(output, tmp, &manager->outputs, link) {
+		output_destroy(output);
+	}
+
+	wl_signal_emit_mutable(&manager->events.destroy, manager);
+
+	assert(wl_list_empty(&manager->events.destroy.listener_list));
+
 	wl_list_remove(&manager->display_destroy.link);
 	wl_list_remove(&manager->layout_add.link);
 	wl_list_remove(&manager->layout_change.link);
@@ -249,20 +260,13 @@ static void handle_display_destroy(struct wl_listener *listener, void *data) {
 
 struct wlr_xdg_output_manager_v1 *wlr_xdg_output_manager_v1_create(
 		struct wl_display *display, struct wlr_output_layout *layout) {
-	// TODO: require wayland-protocols 1.18 and remove this condition
-	int version = OUTPUT_MANAGER_VERSION;
-	if (version > zxdg_output_manager_v1_interface.version) {
-		version = zxdg_output_manager_v1_interface.version;
-	}
-
-	struct wlr_xdg_output_manager_v1 *manager =
-		calloc(1, sizeof(struct wlr_xdg_output_manager_v1));
+	struct wlr_xdg_output_manager_v1 *manager = calloc(1, sizeof(*manager));
 	if (manager == NULL) {
 		return NULL;
 	}
 	manager->layout = layout;
 	manager->global = wl_global_create(display,
-		&zxdg_output_manager_v1_interface, version, manager,
+		&zxdg_output_manager_v1_interface, OUTPUT_MANAGER_VERSION, manager,
 		output_manager_bind);
 	if (!manager->global) {
 		free(manager);
